function [C,k] = gbemult (Cin, M, accum, semiring, A, B, desc) %#ok

% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2025, All Rights Reserved.
% SPDX-License-Identifier: Apache-2.0

error ('GrB:mex', 'mexFunction not found; use gbmake to compile GraphBLAS') ;

