function [lo, hi] = gbbandwidth (G, compute_hi, compute_lo)        %#ok

% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2025, All Rights Reserved.
% SPDX-License-Identifier: Apache-2.0

error ('GrB:mex', 'mexFunction not found; use gbmake to compile GraphBLAS') ;

